@@ -295,12 +295,9 @@ class Cam(dict):
     def project_cam2_depth(cam1, cam2, depth2, T=None, interpolation=1.5):
         if T is None:
             T = cam1.get_T_cam2_in_self(cam2)
-        if interpolation:
-            interpolation_rate = cam1.K[0, 0] / cam2.K[0, 0] * interpolation
-            if interpolation >= 1:
-                interpolation_rate = max(interpolation_rate, 1)
-        else:
-            interpolation_rate = 1
+        interpolation_rate = utils._get_appropriate_interpolation_rate(
+            cam1, cam2, interpolation
+        )
         point_cloud2 = utils.depth_to_point_cloud(
             depth2, cam2.K, interpolation_rate=interpolation_rate
         )
@@ -308,12 +305,39 @@ class Cam(dict):
         depth1 = utils.point_cloud_to_depth(point_cloud1, cam1.K, cam1.xy)
         return depth1
 
-    def vis_project_align(self, img, depth):
+    def vis_depth_alignment(self, img, depth):
+        """
+        Visualize alignment of img and depth using browser
+        Note: img is raw img file without undistort
+        """
         img = cv2.undistort(img, self.K, self.D)
+        depth = depth.clip(0, 5000 if depth.dtype == np.uint16 else 5)
         depth_uint8 = np.uint8(depth / depth.max() * 255)
         depth_vis = np.uint8(cv2.applyColorMap(depth_uint8, cv2.COLORMAP_JET) * 0.75)
         return utils.vis_align(img, depth_vis)
 
+    def vis_reproject_img_alignment(cam1, cam2, depth2, img2, img1, interpolation=1.5):
+        """
+        Visualize alignment of undistorted_img1 and reproject_img from img2
+        Note: img is raw img file without undistort
+        """
+        undistorted_img1 = cv2.undistort(img1, cam1.K, cam1.D)
+        assert not cam2.D.any(), f"cam2.D has distort: {cam2.D}"
+        T_2in1 = cam1.get_T_cam2_in_self(cam2)
+        interpolation_rate = utils._get_appropriate_interpolation_rate(
+            cam1, cam2, interpolation
+        )
+        mapx, mapy = utils.get_reproject_remap(
+            cam1.K,
+            cam2.K,
+            T_2in1,
+            depth2,
+            cam1.xy,
+            interpolation_rate=interpolation_rate,
+        )
+        reproject_img = cv2.remap(img2, mapx, mapy, cv2.INTER_LINEAR)
+        return utils.vis_align(undistorted_img1, reproject_img)
+
     def vis_image_points_cover(self):
         vis = utils._get_vis_background_of_cam(self)
         for uvs in self._get_points_for_cv2():
@@ -497,9 +521,16 @@ class Cam(dict):
             glob(os.path.join(root, "*", "depth_cam_color.jpg")),
             feature_lib,
             name="depth",
-            undistorted=True,
             enable_cache=True,
         )
+        built_in_intrinsics = dict(
+            fx=1474.1182177692722,
+            fy=1474.125874583105,
+            cx=1037.599716850734,
+            cy=758.3072639103259,
+        )
+        # depth need to be used in pairs with camera's built-in intrinsics
+        camd.load(built_in_intrinsics)
         return caml, camr, camd
 
     @classmethod
@@ -525,22 +556,6 @@ class Cam(dict):
         self.xy = xy
         return self
 
-    @classmethod
-    def build_with_multi_feature_libs(cls, imgps, feature_libs, **kwargs):
-        self = cls(**kwargs)
-        if not isinstance(feature_libs, dict):
-            feature_libs = dict([(str(i), v) for i, v in enumerate(feature_libs)])
-        kwargs["save_feature_vis"] = False
-        for name in feature_libs:
-            kwargs["name"] = self.name + "~cam_~" + name
-            feature_lib = feature_libs[name]
-            cam_ = cls(imgps, feature_lib=feature_lib, **kwargs)
-            for k in cam_:
-                cam_[k]["feature_lib"] = feature_lib
-                self[k + "~" + name] = cam_[k]
-        self.calibrate()
-        return self
-
 
 class Cams(list):
     """
@@ -588,6 +603,7 @@ if __name__ == "__main__":
     from boxx import *
 
     caml, camr, camd = Cam.get_test_cams()
+    # caml, camr, camd = utils.get_test_cams("aruco").values()
     print(Cam.load(camd.dump()))
 
     stereo = Stereo(caml, camr)
@@ -597,11 +613,11 @@ if __name__ == "__main__":
     key = caml.valid_keys_intersection(camd)[0]
     imgl = imread(caml[key]["path"])
     color_path_d = camd[key]["path"]
+    imgd = imread(color_path_d)
     depthd = imread(color_path_d.replace("color.jpg", "depth.png"))
     depthd = np.float32(depthd / 1000)
 
-    depthl = caml.project_cam2_depth(camd, depthd, T_camd_in_caml)
-    # depthd_cycle = camd.project_cam2_depth(caml, depthl, camd.get_T_cam2_in_self(caml))
-    # shows(depthd, depthd_cycle)
+    caml.vis_reproject_img_alignment(camd, depthd, imgd, imgl)
 
-    caml.vis_project_align(imgl, depthl)
+    depthl = caml.project_cam2_depth(camd, depthd, T_camd_in_caml)
+    caml.vis_depth_alignment(imgl, depthl)
