@@ -282,14 +282,6 @@ class Stereo:
     def get_max_depth(self):
         return getattr(self, "max_depth", self.MAX_DEPTH)
 
-    def get_calibration_board_T(self, img, feature_lib=None):
-        from calibrating import Cam
-
-        if feature_lib is None:
-            assert hasattr(self.cam1, "feature_lib"), "Please set feature_lib"
-            feature_lib = self.cam1.feature_lib
-        return Cam.get_calibration_board_T(self, img, feature_lib)
-
     @property
     def xy(self):
         return self.cam1.xy
@@ -362,7 +354,7 @@ class Stereo:
         return res
 
     def set_stereo_matching(
-        self, stereo_matching, max_depth=None, translation_rectify_img=False
+        self, stereo_matching, max_depth=None, translation_rectify_img=None
     ):
         """
         Parameters
@@ -371,12 +363,17 @@ class Stereo:
             Subinstance of calibrating.MetaStereoMatching
         max_depth : float, optional
             The default is Stereo.MAX_DEPTH.
-        translation_rectify_img : bool, the default is False.
+        translation_rectify_img : bool, the default is None.
             When self.get_depth(), translation rectify_img2 by self.min_disparity, 
             self.min_disparity is accroding to self.max_depth.
+            default is accroding to bool(max_depth)
         """
         self.stereo_matching = stereo_matching
-        self.translation_rectify_img = translation_rectify_img
+        self.translation_rectify_img = (
+            bool(max_depth)
+            if translation_rectify_img is None
+            else translation_rectify_img
+        )
         self.max_depth = max_depth or self.MAX_DEPTH
         self.min_disparity = int(self.cam1.K[0, 0] * self.baseline / self.max_depth)
 
@@ -489,6 +486,45 @@ class SemiGlobalBlockMatching(MetaStereoMatching):
         return disparity
 
 
+class MatchingByFeatureLib(MetaStereoMatching):
+    """
+    Stereo mathch disp by feature_lib's image_points
+    return a dense disp on calibration board
+    """
+
+    def __init__(self, feature_lib, dense_predict=True):
+        self.feature_lib = feature_lib
+        self.dense_predict = dense_predict
+
+    def __call__(self, img1, img2):
+        self.feature_lib
+        d1 = dict(img=img1)
+        self.feature_lib.find_image_points(d1)
+        image_points1 = d1["image_points"]
+        d2 = dict(img=img2)
+        self.feature_lib.find_image_points(d2)
+        image_points2 = d2["image_points"]
+
+        if isinstance(image_points1, dict):
+            image_points1 = []
+            image_points2 = []
+            for key in sorted(set(d1["image_points"]).intersection(d2["image_points"])):
+                image_points1.append(d1["image_points"][key])
+                image_points2.append(d2["image_points"][key])
+            image_points1 = np.concatenate(image_points1, 0)
+            image_points2 = np.concatenate(image_points2, 0)
+
+        rectify_std = np.std((image_points2 - image_points1)[:, 1])
+        point_disps = (image_points1 - image_points2)[:, 0]
+        xyds = np.append(image_points1, point_disps[:, None], axis=-1)
+        sparse_disp = utils.uvzs_to_arr2d(xyds, img1.shape[:2])
+        if self.dense_predict:
+            dense_disp = utils.interpolate_sparse2d(sparse_disp, "convex_hull")
+            return dense_disp
+        else:
+            return sparse_disp
+
+
 if __name__ == "__main__":
     from boxx import *
     from calibrating import Cam
@@ -505,9 +541,7 @@ if __name__ == "__main__":
     cam1.vis_stereo(cam2, stereo)
 
     stereo_matching = SemiGlobalBlockMatching({})
-    stereo.set_stereo_matching(
-        stereo_matching, max_depth=3.5, translation_rectify_img=True
-    )
+    stereo.set_stereo_matching(stereo_matching, max_depth=3.5)
 
     key = list(cam1)[0]
     img1 = boxx.imread(cam1[key]["path"])
